toolbar_buttons.toolbar_button_loader(toolbar_buttons.interfaces, {
	EffectiveTLDService:Cc["@mozilla.org/network/effective-tld-service;1"].getService(Ci.nsIEffectiveTLDService),
	FilePicker: function() { return Cc['@mozilla.org/filepicker;1'].createInstance(Ci.nsIFilePicker); },
	IOService:Cc['@mozilla.org/network/io-service;1'].getService(Ci.nsIIOService),
	ScriptableInputStream:Cc["@mozilla.org/scriptableinputstream;1"].getService(Ci.nsIScriptableInputStream),
	StringBundleService:Cc['@mozilla.org/intl/stringbundle;1'].getService(Ci.nsIStringBundleService),
	WindowMediator:Cc['@mozilla.org/appshell/window-mediator;1'].getService(Ci.nsIWindowMediator)
});
toolbar_buttons.toolbar_button_loader(toolbar_buttons, {
	handelMenuLoaders: function(event, item) {
		if(item._handelMenuLoaders)
			return;
		item.addEventListener('DOMMenuItemActive', toolbar_buttons.menuLoaderEvent, false);
		item._handelMenuLoaders = true;
	},
	sortMenu: function(event, aMenu) {
		if(aMenu.sorted || !aMenu.getAttribute('sortable')){
			return;
		}
		var menuitems = [];
		while(aMenu.firstChild) {
			menuitems.push(aMenu.firstChild);
			aMenu.removeChild(aMenu.firstChild);
		}
		menuitems.sort(function(a, b) { return a.getAttribute('label').toLowerCase() > b.getAttribute('label').toLowerCase(); });
		for(var i in menuitems) {
			aMenu.appendChild(menuitems[i]);
		}
		aMenu.sorted = true;
	},
	OpenAddonsMgr: function(event, type, typeUrl) {
		var win = event.target.ownerDocument.defaultView;
		var extensionManager = toolbar_buttons.interfaces.WindowMediator
						.getMostRecentWindow("Extension:Manager");
		if (extensionManager) {
			extensionManager.focus();
			extensionManager.showView(type);
		} else {
			var addonManager = toolbar_buttons.interfaces.WindowMediator
				.getMostRecentWindow("Addons:Manager");
			if (addonManager) {
				addonManager.focus();
				addonManager.gViewController.loadView(typeUrl);
			} else {
				var contents = toolbar_buttons.getUrlContents("chrome://mozapps/content/extensions/extensions.xul");
				var ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
	                   .getService(Components.interfaces.nsIWindowWatcher);
				var extWin = ww.openWindow(win, "chrome://mozapps/content/extensions/extensions.xul",
				                        "Addons:Manager", "chrome,centerscreen,width=850,height=550", null);
				extWin.addEventListener("load", function() { extWin.gViewController.loadView(typeUrl); }, false);
			}
		}
	},
	openAddonsExceptions: function(event) {
			toolbar_buttons.openPermissions(event, "install",
					"addons_permissions_title", "addonspermissionstext");
	},
	installAddons: function(event) {
		var doc = event.target.ownerDocument;
		var win = doc.defaultView;
		var stringBundle = toolbar_buttons.interfaces.StringBundleService
				.createBundle("chrome://addons-toolbar-button/locale/addons_button.properties");
		var title = stringBundle.GetStringFromName("installaddons");
		var fp = toolbar_buttons.interfaces.FilePicker();
		fp.init(win, title, fp.modeOpenMultiple);
		fp.appendFilter(stringBundle.GetStringFromName("installaddons-addons"), "*.xpi; *.jar");
		fp.appendFilter(stringBundle.GetStringFromName("installaddons-extensions"), "*.xpi");
		fp.appendFilter(stringBundle.GetStringFromName("installaddons-themes"), "*.jar");
		fp.appendFilters(fp.filterAll);
	
		// taken from /mozilla/toolkit/mozapps/extensions/content/extensions.js
		if (fp.show() != fp.returnOK) {
			return;
		}
		Components.utils.import("resource://gre/modules/AddonManager.jsm");
		
		var files = fp.files;
		var installs = [];
	
		function getBrowserElement() {
			return win.QueryInterface(Ci.nsIInterfaceRequestor)
						.getInterface(Ci.nsIDocShell)
						.chromeEventHandler;
		}
	
		function buildNextInstall() {
			if (!files.hasMoreElements()) {
				if (installs.length > 0) {
					// Display the normal install confirmation for the installs
					AddonManager.installAddonsFromWebpage("application/x-xpinstall", getBrowserElement(), null, installs);
				}
				return;
			}
			var file = files.getNext();
			AddonManager.getInstallForFile(file, function cmd_installFromFile_getInstallForFile(aInstall) {
				installs.push(aInstall);
				buildNextInstall();
			});
		}
	
		buildNextInstall();
	},
	setUpMenuShower: function(doc) {
		var observer = {
			observe: function(aSubject, aTopic, aData) {
				var menu = doc.getElementById(aData);
				if(menu) {
					menu.setAttribute('hidden', !this.prefs.getBoolPref(aData));
				}
			},
			unregister: function() {
				this.prefs.removeObserver("", this);
			},
			register: function() {toolbar_buttons.registerCleanUpFunction(this.unregister);
				this.prefs = Cc['@mozilla.org/preferences-service;1'].getService(Ci.nsIPrefService).getBranch("extension.tbutton.showamenu.");
				this.prefs.addObserver("", this, false);
				var self = this;
				toolbar_buttons.registerCleanUpFunction(function() {
					self.prefs.removeObserver("", self);
				});
				var attrList = this.prefs.getChildList('', {});
				for(var i in attrList) {
					var menu = doc.getElementById(attrList[i]);
					if(menu) {
						menu.setAttribute('hidden', !this.prefs.getBoolPref(attrList[i]));
					}
				}
			}
		};
		observer.register();
	},
	menuLoaderEvent: function(event) {
		var win = event.target.ownerDocument.defaultView;
		var menuitem = event.originalTarget;
		if(menuitem.getAttribute('showamenu')) {
			// so this is one of those menu items that as a fake submenu, show it
			var cEvent = new Event('command', {
			    'view': win,
			    'bubbles': false,
			    'cancelable': true,
			    'target': menuitem,
			});
			menuitem.dispatchEvent(cEvent);
		}
	},
	getUrlContents: function(aURL){
		var ioService = toolbar_buttons.interfaces.IOService;
		var scriptableStream = toolbar_buttons.interfaces.ScriptableInputStream;
		var channel=ioService.newChannel(aURL,null,null);
		var input = channel.open();
		scriptableStream.init(input);
		var str = scriptableStream.read(input.available());
		scriptableStream.close();
		input.close();
		return str;
	},
	openPermissions: function(event, type, title, text) {
		var doc = event.target.ownerDocument;
		var win = doc.defaultView;
		var bundlePreferences = toolbar_buttons.interfaces.StringBundleService
			.createBundle("chrome://browser/locale/preferences/preferences.properties");
		var params = { blockVisible   : true,
					   sessionVisible : true,
					   allowVisible   : true,
					   prefilledHost  : toolbar_buttons.getETDL(event),
					   permissionType : type,
					   windowTitle	: bundlePreferences.GetStringFromName(title),
					   introText	  : bundlePreferences.GetStringFromName(text) };
		win.openDialog("chrome://browser/content/preferences/permissions.xul",
				"Browser:Permissions", "", params);
	},
	getETDL: function(event) {
		var win = event.target.ownerDocument.defaultView;
		var eTLDService = toolbar_buttons.interfaces.EffectiveTLDService;
	
		var eTLD;
		var uri = win.content.document.documentURIObject;
		try {
			eTLD = eTLDService.getBaseDomain(uri);
		} catch (e) {
			// getBaseDomain will fail if the host is an IP address or is empty
			eTLD = uri.asciiHost;
		}
		return eTLD;
	}
});
